@@ -25,7 +25,11 @@ impl WireguardDeviceLinux {
         mut args: Vec<&str>,
         input: Option<&str>,
     ) -> BoxResult<std::process::Output> {
-        let mut args_with_sudo = vec!["sudo"];
+        let mut args_with_sudo = vec![];
+        if !nix::unistd::getuid().is_root() {
+            args_with_sudo.push("sudo");
+            args_with_sudo.push("WG_I_PREFER_BUGGY_USERSPACE_TO_POLISHED_KMOD=1")
+        }
         args_with_sudo.append(&mut args);
 
         let stdin_par = if input.is_none() {
@@ -36,6 +40,7 @@ impl WireguardDeviceLinux {
 
         let child = Command::new(args_with_sudo.remove(0))
             .args(args_with_sudo)
+            .env("WG_I_PREFER_BUGGY_USERSPACE_TO_POLISHED_KMOD", "1")
             .stdin(stdin_par)
             .stdout(Stdio::piped())
             .stderr(Stdio::piped())
@@ -93,6 +98,9 @@ impl WireguardDevice for WireguardDeviceLinux {
         Ok(result.is_ok())
     }
     fn create_device(&self) -> BoxResult<()> {
+        //let kernel_unicast = netlink_sys::SocketAddr::new(0, 0);
+        //let socket = netlink_sys::Socket::new(netlink_sys::protocols::NETLINK_AUDIT)?;
+
         debug!("Create device via ip link add");
         let mut result = self.execute_command(
             vec!["ip", "link", "add", &self.device_name, "type", "wireguard"],
@@ -103,14 +111,7 @@ impl WireguardDevice for WireguardDeviceLinux {
             // try wireguard-go
             debug!("Create device via wireguard-go");
             // Setting the environment variable works only with sudo
-            result = self.execute_command(
-                vec![
-                    "WG_I_PREFER_BUGGY_USERSPACE_TO_POLISHED_KMOD=1",
-                    "wireguard-go",
-                    &self.device_name,
-                ],
-                None,
-            );
+            result = self.execute_command(vec!["wireguard-go", &self.device_name], None);
         }
 
         if result.is_err() {
@@ -256,9 +257,12 @@ impl WireguardDevice for WireguardDeviceLinux {
         for peer_ini in ini.section_all(Some("Peer")) {
             if let Some(endpoint) = peer_ini.get("Endpoint") {
                 if let Some(pub_key) = peer_ini.get("PublicKey") {
-                    let sock_addr: SocketAddr = endpoint.parse().unwrap();
-                    trace!("{} is endpoint of {}", sock_addr, pub_key);
-                    pubkey_to_endpoint.insert(pub_key.to_string(), sock_addr);
+                    if let Ok(sa) = v6_strip_interface(endpoint) {
+                        if let Ok(sock_addr) = sa.parse::<SocketAddr>() {
+                            trace!("{} is endpoint of {}", sock_addr, pub_key);
+                            pubkey_to_endpoint.insert(pub_key.to_string(), sock_addr);
+                        }
+                    }
                 }
             }
         }
