@@ -41,7 +41,7 @@ fn get_option_string(
 
 fn main() -> BoxResult<()> {
     let matches = App::new("Wireguard Network Manager")
-        .version("0.3.8")
+        .version(env!("CARGO_PKG_VERSION"))
         .author("Jochen Kiemes <jochen@kiemes.de>")
         .about("Manages a network of wireguard nodes with no central server.")
         .arg(
@@ -134,15 +134,27 @@ fn main() -> BoxResult<()> {
     let use_tui = matches.is_present("tui");
 
     let mut opt_peer_conf: Option<Yaml> = None;
+    // unwrap() is ok here due to the default value in clap
     let peer_config = matches.value_of("peer_config").unwrap();
-    if let Ok(mut file) = File::open(peer_config) {
-        let mut content = String::new();
-        file.read_to_string(&mut content)?;
-        let mut peer_conf = YamlLoader::load_from_str(&content).unwrap();
-        if peer_conf.len() != 1 {
-            return Err("Malformed peer configuration".into());
+    match File::open(peer_config) {
+        Ok(mut file) => {
+            let mut content = String::new();
+            file.read_to_string(&mut content)?;
+            let mut peer_conf_vec = YamlLoader::load_from_str(&content).unwrap();
+            if peer_conf_vec.len() != 1 {
+                return Err("Malformed peer configuration".into());
+            }
+            opt_peer_conf = Some(peer_conf_vec.remove(0));
         }
-        opt_peer_conf = Some(peer_conf.remove(0));
+        Err(e) => match e.kind() {
+            std::io::ErrorKind::PermissionDenied => {
+                return Err(format!("Permission denied for {}", peer_config).into());
+            }
+            std::io::ErrorKind::NotFound => {
+                // That's OK
+            }
+            _ => return Err(Box::new(e)),
+        },
     }
 
     let computer_name = get_option_string(&matches, &opt_peer_conf, "name")?;
@@ -173,43 +185,80 @@ fn main() -> BoxResult<()> {
     }
 
     let network_config = matches.value_of("network_config").unwrap();
-    let mut file = File::open(network_config)?;
-    let mut content = String::new();
-    file.read_to_string(&mut content)?;
-    let network_conf_vec = YamlLoader::load_from_str(&content).unwrap();
-    if network_conf_vec.len() != 1 {
-        return Err("Malformed network configuration".into());
+    let network_conf: Yaml;
+    match File::open(network_config) {
+        Ok(mut file) => {
+            let mut content = String::new();
+            file.read_to_string(&mut content)?;
+            let mut network_conf_vec = YamlLoader::load_from_str(&content).unwrap();
+            if network_conf_vec.len() != 1 {
+                return Err("Malformed network configuration".into());
+            }
+            network_conf = network_conf_vec.remove(0);
+            debug!("Raw configuration:");
+            debug!("{:#?}", network_conf);
+        }
+        Err(e) => match e.kind() {
+            std::io::ErrorKind::PermissionDenied => {
+                return Err(format!("Permission denied for {}", network_config).into());
+            }
+            std::io::ErrorKind::NotFound => {
+                return Err(format!(
+                    "Cannot find required network configuration: {}",
+                    network_config
+                )
+                .into());
+            }
+            _ => return Err(Box::new(e)),
+        },
     }
-    let network_conf = &network_conf_vec[0];
-    debug!("Raw configuration:");
-    debug!("{:#?}", network_conf);
 
     let ip_list = Arch::get_local_interfaces();
 
     let use_existing_interface = get_option_bool(&matches, &opt_peer_conf, "existingInterface");
     let interface = get_option_string(&matches, &opt_peer_conf, "wgInterface")?;
     let wg_ip_string = get_option_string(&matches, &opt_peer_conf, "wgIp")?;
-    let wg_ip: Ipv4Addr = wg_ip_string.parse().unwrap();
-    let wg_port: u16 = matches.value_of("wireguard_port").unwrap().parse().unwrap();
-    let admin_port: u16 = matches.value_of("admin_port").unwrap().parse().unwrap();
+    let wg_ip: Ipv4Addr = wg_ip_string.parse()?;
+
+    // Due to default values in clap, the unwraps() before parse() are ok
+    let wg_port: u16 = matches.value_of("wireguard_port").unwrap().parse()?;
+    let admin_port: u16 = matches.value_of("admin_port").unwrap().parse()?;
 
     let network = &network_conf["network"];
-    let shared_key = base64::decode(&network["sharedKey"].as_str().unwrap()).unwrap();
-    let subnet: ipnet::Ipv4Net = network["subnet"].as_str().unwrap().parse().unwrap();
+    let shared_key = base64::decode(
+        &network["sharedKey"]
+            .as_str()
+            .ok_or("sharedKey is not defined or not a string")?,
+    )?;
+    let subnet: ipnet::Ipv4Net = network["subnet"]
+        .as_str()
+        .ok_or("subnet is not defined or not a string")?
+        .parse()?;
 
     if !subnet.contains(&wg_ip) {
-        return Err(format!("{} is outside of {}", wg_ip, subnet).into());
+        return Err(format!("{} is outside of subnet {}", wg_ip, subnet).into());
     }
 
     let mut peers: HashMap<Ipv4Addr, PublicPeer> = HashMap::new();
-    for p in network_conf["peers"].as_vec().unwrap() {
+    for p in network_conf["peers"]
+        .as_vec()
+        .ok_or("no peers defined in config file")?
+    {
         info!("STATIC PEER: {:#?}", p);
-        let endpoint = p["endPoint"].as_str().unwrap().to_string();
+        let endpoint = p["endPoint"]
+            .as_str()
+            .ok_or("no endpoint defined")?
+            .to_string();
         let mut flds = endpoint.split(':').collect::<Vec<_>>();
-        let port_str = flds.pop().unwrap();
-        let wg_port = (*port_str).parse::<u16>().unwrap();
-        let admin_port = p["adminPort"].as_i64().unwrap() as u16;
-        let wg_ip: Ipv4Addr = p["wgIp"].as_str().unwrap().parse().unwrap();
+        let port_str = flds.pop().ok_or("endpoint should be <hostname/ip:port>")?;
+        let wg_port = (*port_str).parse::<u16>()?;
+        let admin_port = p["adminPort"]
+            .as_i64()
+            .ok_or("Cannot parse adminPort as integer")? as u16;
+        let wg_ip: Ipv4Addr = p["wgIp"]
+            .as_str()
+            .ok_or("wgIp not defined or not a string")?
+            .parse()?;
         let pp = PublicPeer {
             endpoint,
             admin_port,
@@ -257,10 +306,5 @@ fn main() -> BoxResult<()> {
         return Ok(());
     }
 
-    //if let Some(("install", cmd)) = subcommand {
-    //    println!("found install");
-    //    return Ok(());
-    //}
-
     wg_netmanager::run_loop::run(&static_config, wg_dev)
 }
